/****************************************************************************
** $Id: qt/qmemarray.h   3.3.4   edited May 27 2003 $
**
** Definition of QMemArray template/macro class
**
** Created : 930906
**
** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
**
** This file is part of the tools module of the Qt GUI Toolkit.
**
** This file may be distributed under the terms of the Q Public License
** as defined by Trolltech AS of Norway and appearing in the file
** LICENSE.QPL included in the packaging of this file.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding valid Qt Enterprise Edition or Qt Professional Edition
** licenses may use this file in accordance with the Qt Commercial License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
**   information about Qt Commercial License Agreements.
** See http://www.trolltech.com/qpl/ for QPL licensing information.
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/

#ifndef QMEMARRAY_H
#define QMEMARRAY_H

#include <iostream>
using namespace std;

#ifndef QT_H
#include "qgarray.h"
#endif // QT_H


template<class T>
class QMemArray : public QGArray
{
public:
    typedef T* Iterator;
    typedef const T* ConstIterator;
    typedef T ValueType;

protected:
    QMemArray( int, int ) : QGArray( 0, 0 ) {}

public:
    explicit QMemArray() : QGArray() { }
    explicit QMemArray(int size) : QGArray(size*sizeof(T)) {} // ### 4.0 Q_EXPLICIT
    QMemArray( const QMemArray<T> &a ) : QGArray(a) {}
    virtual ~QMemArray() {}
    QMemArray<T>& operator= (const QMemArray<T> &a)
	{ return reinterpret_cast<QMemArray<T>&>(QGArray::assign(a)); }
    T* data() const
    {
	volatile T* vol = reinterpret_cast<volatile T*>(QGArray::data());
	register T* ret = (T *) vol;
	return reinterpret_cast<T*>(ret);
    }
    uint  nrefs()   const	{ return QGArray::nrefs(); }
    uint  size()    const	{ return QGArray::size()/sizeof(T); }
    uint  count()   const	{ return size(); }
    bool  isEmpty() const	{ return QGArray::size() == 0; }
    bool  isNull()  const	{ return QGArray::data() == 0; }
    bool  resize( uint size )	{ return QGArray::resize(size*sizeof(T)); }
    bool  resize( uint size, Optimization optim ) { return QGArray::resize(size*sizeof(T), optim); }
    bool  truncate( uint pos )	{ return QGArray::resize(pos*sizeof(T)); }
    bool  fill( const T &d, int size = -1 )
	{ return QGArray::fill(reinterpret_cast<const char*>(&d), size, sizeof(T) ); }
    void  detach() { QGArray::detach(); }
    QMemArray<T>   copy() const
	{ QMemArray<T> tmp; return tmp.duplicate(*this); }
    QMemArray<T>& assign( const QMemArray<T>& a )
	{ return (reinterpret_cast<QMemArray<T>&>(QGArray::assign(a))); }
    QMemArray<T>& assign( const T *a, uint n )
	{ return (reinterpret_cast<QMemArray<T>&>(QGArray::assign(reinterpret_cast<const char*>(a), n * sizeof(T)))); }
    QMemArray<T>& duplicate( const QMemArray<T>& a )
	{ return (reinterpret_cast<QMemArray<T>&>(QGArray::duplicate(a))); }
    QMemArray<T>& duplicate( const T *a, uint n )
	{ return (reinterpret_cast<QMemArray<T>&>(QGArray::duplicate(reinterpret_cast<const char*>(a), n * sizeof(T)))); }
    QMemArray<T>& setRawData( const T *a, uint n )
	{ return (reinterpret_cast<QMemArray<T>&>(QGArray::setRawData(reinterpret_cast<const char*>(a), n * sizeof(T)))); }
    void resetRawData( const T *a, uint n )
	{ QGArray::resetRawData(reinterpret_cast<const char*>(a), n * sizeof(T)); }
    int	 find( const T &d, uint i=0 ) const
	{ return QGArray::find(reinterpret_cast<const char*>(&d), i, sizeof(T)); }
    int	 contains( const T &d ) const
	{ return QGArray::contains(reinterpret_cast<const char*>(&d), sizeof(T)); }
    void sort() { QGArray::sort(sizeof(T)); }
    int  bsearch( const T &d ) const
	{ return QGArray::bsearch(reinterpret_cast<const char*>(&d), sizeof(T)); }
    // ### Qt 4.0: maybe provide uint overload as work-around for MSVC bug
    T& operator[]( int i ) const
	{ return (reinterpret_cast<T&>(*(reinterpret_cast<T*>(QGArray::at(i*sizeof(T)))))); }
    T& at( uint i ) const
	{ return (reinterpret_cast<T&>(*(reinterpret_cast<T*>(QGArray::at(i*sizeof(T)))))); }
	 operator const T*() const { return (reinterpret_cast<const T*>(QGArray::data())); }
    bool operator==( const QMemArray<T> &a ) const { return isEqual(a); }
    bool operator!=( const QMemArray<T> &a ) const { return !isEqual(a); }
    Iterator begin() { return data(); }
    Iterator end() { return data() + size(); }
    ConstIterator begin() const { return data(); }
    ConstIterator end() const { return data() + size(); }
};

#ifndef QT_NO_COMPAT
#define QArray QMemArray
#endif

#define Q_DEFINED_QMEMARRAY
#include "qwinexport.h"
#endif // QARRAY_H
